import pytest
import torch
from homa.models import Resnet, Model
from homa.models.modules import ResnetModule


@pytest.fixture
def resnet_model():
    model = Resnet(num_classes=5, lr=0.001)
    return model


def test_resnet_initialization(resnet_model):
    assert isinstance(resnet_model, Resnet)
    assert isinstance(resnet_model, Model)
    assert hasattr(resnet_model, "network")
    assert hasattr(resnet_model, "optimizer")
    assert hasattr(resnet_model, "criterion")
    assert isinstance(resnet_model.network, ResnetModule)
    assert isinstance(resnet_model.optimizer, torch.optim.SGD)
    assert isinstance(resnet_model.criterion, torch.nn.CrossEntropyLoss)



import torch
from .modules import ResnetModule
from .Model import Model


class Resnet(Model):
    def __init__(self, num_classes: int, lr: float):
        super().__init__()
        self.network = ResnetModule(num_classes)
        self.optimizer = torch.optim.SGD(self.network.parameters(), lr=lr, momentum=0.9)
        self.criterion = torch.nn.CrossEntropyLoss()

    def train(self, x: torch.Tensor, y: torch.Tensor):
        self.network.train()
        self.optimizer.zero_grad()
        loss = self.criterion(x, y)
        loss.backward()
        self.optimizer.step()
